﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;

namespace ComplexConverter
{
    public class EventArg : EventArgs
    {
        // Das Event ComplexConverter.Convert(), welches dieses EventArg verwendet, wird in zwei
        // Modi ausgeführt:
        // Store konvertiert einen gegebenen Wert und fügt der IList(Of String) einen Eintrag hinzu
        // Restore re-konvertiert einen Eintrag und überschreibt damit den gegebenen Wert
        private IList<string> _Strings;
        private int _Counter;
        public readonly bool IsStoring;

        public EventArg(IList<string> Strings, bool IsStoring)
        {
            this._Counter = 0;
            this._Strings = Strings;
            this.IsStoring = IsStoring;
        }

        /// <summary>
        /// Hier angegebene Werte werden gespeichert und restauriert
        /// </summary>
        [Description("Hier angegebene Werte werden gespeichert und restauriert")]
        public void ConvertValue<T>(ref T arg)
        {
            object o = arg;
            this.ConvertExplicitTyped<T>(ref o);
            arg = (T)o;
        }

        /// <summary>
        /// Hier angegebene Werte werden gespeichert und restauriert
        /// </summary>
        [Description("Hier angegebene Werte werden gespeichert und restauriert")]
        public void ConvertValue<T1, T2>(ref T1 arg1, ref T2 arg2)
        {
            object o = arg1;
            this.ConvertExplicitTyped<T1>(ref o);
            arg1 = (T1)o;
            o = arg2;
            this.ConvertExplicitTyped<T2>(ref o);
            arg2 = (T2)o;
        }

        /// <summary>
        /// Hier angegebene Werte werden gespeichert und restauriert
        /// </summary>
        [Description("Hier angegebene Werte werden gespeichert und restauriert")]
        public void ConvertValue<T1, T2, T3>(ref T1 arg1, ref T2 arg2, ref T3 arg3)
        {
            object o = arg1;
            this.ConvertExplicitTyped<T1>(ref o);
            arg1 = (T1)o;
            o = arg2;
            this.ConvertExplicitTyped<T2>(ref o);
            arg2 = (T2)o;
            o = arg3;
            this.ConvertExplicitTyped<T3>(ref o);
            arg3 = (T3)o;
        }

        /// <summary>
        /// Hier angegebene Werte werden gespeichert und restauriert
        /// </summary>
        [Description("Hier angegebene Werte werden gespeichert und restauriert")]
        public void ConvertValue<T1, T2, T3, T4>(ref T1 arg1, ref T2 arg2, ref T3 arg3, ref T4 arg4)
        {
            object o = arg1;
            this.ConvertExplicitTyped<T1>(ref o);
            arg1 = (T1)o;
            o = arg2;
            this.ConvertExplicitTyped<T2>(ref o);
            arg2 = (T2)o;
            o = arg3;
            this.ConvertExplicitTyped<T3>(ref o);
            arg3 = (T3)o;
            o = arg4;
            this.ConvertExplicitTyped<T4>(ref o);
            arg4 = (T4)o;
        }

        /// <summary>
        /// Hier angegebene Werte werden gespeichert und restauriert.
        /// Der Datentyp ist anzugeben
        /// </summary>
        [Description("Hier angegebene Werte werden gespeichert und restauriert")]
        public void ConvertExplicitTyped<T>(ref object item)
        {
            if (this.IsStoring)
            {
                string s = item as string;
                if (s == null)
                {
                    s = TypeDescriptor.GetConverter(typeof(T)).ConvertToString(item);
                }
                else if (s.Contains(ComplexConverter.Separator.ToString()))
                {
                    string msg = "Der zu speichernde String '" + s + "' darf nicht den ComplexConverter.Separator-Char (" + System.Convert.ToString(124, 16) + ") enthalten.";
                    throw new ArgumentException(msg, "Item");
                }

                this._Strings.Add(s);
            }
            else
            {
                item = this.GetValue<T>();
            }
        }

        /// <summary>
        /// gibt einen gespeicherten Wert zurück
        /// </summary>
        [Description("gibt einen gespeicherten Wert zurück")]
        public T GetValue<T>()
        {
            if (this.IsStoring)
            {
                throw new ComplexConverterException(".GetValue() darf nicht aufgerufen werden, wenn .IsStoring = True.", null);
            }
            
            TypeConverter C = TypeDescriptor.GetConverter(typeof(T));
            T GetValue = (T)C.ConvertFromString(this._Strings[this._Counter]);

            this._Counter++;
            return GetValue;
        }

        // Sonderfall ConvertList(): Bei Listen wird die Anzahl Count der Elemente gespeichert.
        // Restore leert die Liste und fügt dann Count uninitialisierte Elemente hinzu.
        /// <summary>
        /// speichert Anzahl, restauriert mit uninitialisierten Elementen
        /// </summary>
        [Description("speichert Anzahl, restauriert mit uninitialisierten Listenelementen")]
        public void ConvertList<TItem>(IList list) where TItem : new()
        {
            if (this.IsStoring)
            {
                this._Strings.Add(list.Count.ToString());
                return;
            }

            list.Clear();
            for (int i = 0; i < int.Parse(this._Strings[this._Counter]); i++)
            {
                list.Add(new TItem());
            }

            this._Counter++;
        }

        public void ConvertConverter(ComplexConverter Converter)
        {
            // ComplexConverters listigste Zeile: ermöglicht, den Konvertierungs-Vorgang an einen
            // anderen Converter abzugeben, dabei aber "meine" _Strings zu verwenden. So kann ein
            // UserControl aus einer anderen Assembly seine eigene Konvertierung definieren, die
            // dann in den Settings der Entry-Assembly mit-persistiert wird
            Converter.OnConvert(this);
        }
    }
}
