﻿using System;
using System.Drawing;
using System.Windows.Forms;

namespace ComplexConverter
{
    using Settings = Properties.Settings;

    public partial class Form1 : Form
    {
        private ComplexConverter _Memory;

        public Form1()
        {
            this.InitializeComponent();

            this._Memory = new ComplexConverter();
            this._Memory.Convert += new EventHandler<EventArg>(this._Memory_Convert);
        }

        /// <summary>
        /// komplexe Konvertierung für dieses Form
        /// </summary>
        private void _Memory_Convert(object sender, EventArg e)
        {
            // TODO: Hier muss die komplette Form angehandelt werden

            // Form-Properties konvertieren
            FormWindowState windowState = this.WindowState;
            e.ConvertValue(ref windowState);
            this.WindowState = windowState;

            Rectangle bounds = this.Bounds;
            e.ConvertValue(ref bounds);
            this.Bounds = bounds;

            // Treeview konvertieren
            this.EnumerateNodes(this.TreeView1.Nodes, e);

            foreach (ColumnHeader Col in this.ListView1.Columns)
            {
                int width = Col.Width;
                e.ConvertValue(ref width);
                Col.Width = width;
            }

            // ListView-Items konvertieren
            e.ConvertList<ListViewItem>(this.ListView1.Items);
            foreach (ListViewItem LVI in this.ListView1.Items)
            {
                // ListView-SubItems konvertieren
                e.ConvertList<ListViewItem.ListViewSubItem>(LVI.SubItems);
                foreach (ListViewItem.ListViewSubItem LVSI in LVI.SubItems)
                {
                    string text = LVSI.Text;
                    e.ConvertValue(ref text);
                    LVSI.Text = text;
                }
            }
        }

        private void EnumerateNodes(TreeNodeCollection nodes, EventArg e)
        {
            e.ConvertList<TreeNode>(nodes);
            foreach (TreeNode nd in nodes)
            {
                string text = nd.Text;
                e.ConvertValue(ref text);
                nd.Text = text;
                this.EnumerateNodes(nd.Nodes, e);
                // Workaround, da Treenode.IsExpanded readonly,
                // also nicht direkt restaurierbar
                if (e.IsStoring)
                {
                    bool isExpanded = nd.IsExpanded;
                    e.ConvertValue(ref isExpanded);
                }
                else if (e.GetValue<bool>())
                {
                    nd.Expand();
                }
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            // Settings.Default.Memory ist in den Anwendungseinstellungen eingerichtet
            this._Memory.ApplyDataString(Settings.Default.Memory);
            this.RichTextBox1.Text = Settings.Default.Memory;
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            Settings.Default.Memory = this._Memory.CreateDataString();
        }

        private void Button_Click(object sender, EventArgs e)
        {
            if (sender.Equals(this.btRead))
            {
                this._Memory.ApplyDataString(this.RichTextBox1.Text);
            }
            else if (sender.Equals(this.btWrite))
            {
                this.RichTextBox1.Text = this._Memory.CreateDataString();
            }
            else if (sender.Equals(this.btAddNode))
            {
                TreeNode Nd2 = this.TreeView1.SelectedNode;
                if (Nd2 == null)
                {
                    this.TreeView1.Nodes.Add("NewNode");
                }
                else
                {
                    Nd2.Nodes.Add("NewNode");
                    Nd2.Expand();
                }
            }
            else if (sender.Equals(this.btRemoveNode))
            {
                TreeNode Nd = this.TreeView1.SelectedNode;
                if (Nd == null)
                {
                    MessageBox.Show("Select a Node to remove");
                }
                else
                {
                    Nd.Remove();
                }
            }
            else if (sender.Equals(this.btAddLVItem))
            {
                int N = this.ListView1.Items.Count;
                ListViewItem LVI = this.ListView1.Items.Add(string.Concat(N, 1));
                for (int i = 2; i <= this.ListView1.Columns.Count; i++)
                {
                    LVI.SubItems.Add(string.Concat(N, i));
                }
            }
            else if (sender.Equals(this.btRemoveListViewItem))
            {
                if (this.ListView1.SelectedItems.Count == 0)
                {
                    MessageBox.Show("Select a ListViewItem to remove");
                }
                else
                {
                    this.ListView1.Items.RemoveAt(this.ListView1.SelectedItems[0].Index);
                }
            }
        }
    }
}
