﻿using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;

namespace ComplexConverter
{
    using Settings = Properties.Settings;

    public partial class FormDgv : Form
    {
        private ComplexConverter _Memory;

        public FormDgv()
        {
            this.InitializeComponent();

            this._Memory = new ComplexConverter();
            this._Memory.Convert += new EventHandler<EventArg>(this._Memory_Convert);

            this.Dgv.ColumnCount = 8;
            if (!this._Memory.ApplyDataString(Settings.Default.DgvMemory))
            {
                this.Dgv.RowCount = 6;
                for (int x = 0; x < this.Dgv.ColumnCount; x++)
                {
                    for (int y = 0; y < this.Dgv.RowCount - 1; y++)
                    {
                        this.Dgv[x, y].Value = (y + x * 10).ToString("00");
                    }
                }
            }
        }

        /// <summary>
        /// komplexe Konvertierung für dieses Form
        /// </summary>
        private void _Memory_Convert(object sender, EventArg e)
        {
            // TODO: Hier muss die komplette Form angehandelt werden
            Rectangle bounds = this.Bounds;
            e.ConvertValue(ref bounds);
            this.Bounds = bounds;

            int num = this.Dgv.RowCount;
            e.ConvertValue(ref num);
            this.Dgv.RowCount = num;

            foreach (DataGridViewColumn col in this.Dgv.Columns)
            {
                int index = col.DisplayIndex;
                int width = col.Width;
                string txt = col.HeaderText;
                // bis zu 4 Werte in eim Aufruf persisteable
                e.ConvertValue(ref index, ref width, ref txt);
                col.DisplayIndex = index;
                col.Width = width;
                col.HeaderText = txt;
            }

            foreach (DataGridViewRow row in this.Dgv.Rows)
            {
                int height = row.Height;
                e.ConvertValue(ref height);
                row.Height = height;
            }

            for (int x = 0; x < this.Dgv.ColumnCount; x++)
            {
                for (int y = 0; y < this.Dgv.RowCount - 1; y++)
                {
                    object obj = this.Dgv[x, y].Value;
                    e.ConvertExplicitTyped<string>(ref obj);
                    this.Dgv[x, y].Value = obj;
                }
            }


            SortOrder order = this.Dgv.SortOrder;
            int sortCol;
            sortCol = (order == SortOrder.None) ? -1 : this.Dgv.SortedColumn.Index;
            e.ConvertValue(ref order, ref sortCol);

            if (!e.IsStoring && order != 0)
            {
                this.Dgv.Sort(this.Dgv.Columns[sortCol], (ListSortDirection)(order - 1));
            }
        }

        private void FormDgv_FormClosing(object sender, FormClosingEventArgs e)
        {
            Settings.Default.DgvMemory = this._Memory.CreateDataString();
        }

        private void btClearSettings_Click(object sender, EventArgs e)
        {
            Settings.Default.DgvMemory = null;
            this.FormClosing -= this.FormDgv_FormClosing;
            System.Media.SystemSounds.Asterisk.Play();
        }
    }
}
